library(ggplot2)

incmean <- function(val, oldmean, n) {
    ## calculate incremental mean
    return(oldmean + (val - oldmean)/n)
}

mycomb <- function(obj) {
  ids <- which(diff(rank(obj$obs)) < 0)
  domore <- (length(ids) > 0)
  while(domore) {
    theid <- ids[1]
    obj$nobs[theid] <- obj$nobs[theid] + 1
    
    obj$obs[theid] <- incmean(obj$obs[theid + 1],
                              obj$obs[theid],
                              obj$nobs[theid] )
    obj$obs[theid + 1] <- NA
    obj$nobs <- obj$nobs[is.finite(obj$obs)]
    obj$obs <- obj$obs[is.finite(obj$obs)]
    ids <- which(diff(rank(obj$obs)) < 0)
    domore <- (length(ids) > 0)
  }
  return(obj)
}

mytidy <- function(x) {
  obj <- list(obs=x,nobs=rep(1,length(x)))
  out <- mycomb(obj)$obs
  ret <- rep(NA,length(x))
  ret[1:length(out)] <- out
  return(ret)
}

lseq <- function (from, to, length) 
{
  ## I copied this out of sfsmisc
  stopifnot(from > 0)
  exp(seq(log(from), log(to), length.out = length))
}

pdfgauss <- function(De, sigma, log.scale=FALSE, nsamp=500)
{
  ## Purpose: Get the summed Guassian distribution
  ## ----------------------------------------------------------------------
  ## Arguments: De and sd(De) ie sigma
  ## ----------------------------------------------------------------------
  ## Author: Robert, Date: 22 Mar 2013, 12:08
  ## guess the required range
  minid <- which.min(De)
  maxid <- which.max(De)
  xmax <- De[maxid] + 3*sigma[maxid]
  xmin <- De[minid] - 3*sigma[minid]
  if(log.scale) {
    xmin <- ifelse(xmin<=0,0.001,xmin)
    x <- lseq(xmin, xmax, length=nsamp)
  } else {
    x <- seq(xmin, xmax, length=nsamp)
  }
  nobs <- length(De)
  y.m <- matrix(NA,ncol=nobs,nrow=nsamp)
  for(i in 1:nobs) {
    y.m[,i] <- dnorm(x,De[i],sigma[i])
  }
  y <- rowSums(y.m)
  return(list(x=x,y=y, De=De, sigma=sigma))
}

pdfgaussplot <- function(obj, meanprob)
{
    ## Purpose: A ggplot of the output from pdfgauss
    ## ----------------------------------------------------------------------
    ## Arguments: obj, the output of pdfgauss
    ##            meanprob, the mean probability line. Typically
    ##                    this is left out and will be calculated
    ##                    from the data.
    ## ----------------------------------------------------------------------
    ## Author: Robert, Date: 22 Mar 2013, 13:21
    aa.df <- data.frame(x = obj$x, y = obj$y)
    ## apply a simple scaling so that maximum
    ## is 1.
    ## need to work out how to show as polygon.
    aa.df <- transform(aa.df, yscaled=y/(max(y, na.rm=TRUE)))
    aa.df <- rbind(data.frame(x=aa.df$x[1],y=NA, yscaled=0),
                   aa.df, 
                   data.frame(x=aa.df$x[1],y=NA, yscaled=0))
    bb.df <- data.frame(De = obj$De, sigma = obj$sigma)
    bb.df <- bb.df[rev(order(bb.df$De)), ]
    bb.df <- transform(bb.df, z = seq(min(aa.df$yscaled)+0.02, max(aa.df$yscaled)-0.02, 
        length = nrow(bb.df)))
    ## and add polygons
    if(missing(meanprob)) {
        meanprob <- mean(aa.df$yscaled, na.rm=TRUE)
    }
    print(meanprob)
    above <- aa.df$yscaled >= meanprob
    sections <- rle(above)
    sectionids <- which(sections$values)
    
    polys.df <- data.frame()
    thisid <- 1
    for(chunk in sectionids) {
        startid <- sum(sections$lengths[1:(chunk-1)]) + 1
        endid <- startid + sections$lengths[chunk]
        xobs <- c(aa.df$x[startid-1],aa.df$x[startid])
        yobs <- c(aa.df$yscaled[startid-1],aa.df$yscaled[startid])
        x1 <- approx(yobs, xobs, xout=meanprob)$y
        ## and second point
        xobs <- c(aa.df$x[endid-1],aa.df$x[endid])
        yobs <- c(aa.df$yscaled[endid-1],aa.df$yscaled[endid])
        x2 <- approx(yobs,xobs, xout=meanprob)$y
        poly.df <- data.frame(id=thisid, 
                              x=c(x1,x2,x2,x1),
                              y=c(0, 0, Inf, Inf))
        polys.df <- rbind(polys.df, poly.df)
        thisid <- thisid+1
    }
    p1 <- ggplot() + 
        geom_polygon(data=polys.df,aes(x=x,y=y, group=id), colour=grey(0.9), alpha=0.1) + 
        geom_errorbarh(data = bb.df, aes(x = De, xmax = De + 
            sigma, xmin = De - sigma, y = z, height = 0), colour=grey(0.7)) + 
        geom_point(data = bb.df, 
                   aes(x = De, y = z), size = 2, shape = 21, colour=grey(0.7), fill=grey(0.7)) +
        geom_line(data = aa.df, aes(x = x, y = yscaled)) +
        geom_polygon(data=aa.df, aes(x=x, y=yscaled), fill=grey(0.5))
    return(invisible(p1))
}


eventfreq <- function(age, ...) UseMethod("eventfreq")

eventfreq.default <- function(age, error, nsim,...) {
    nobs <- length(age)
    ## sort the ages
    error <- error[order(age)]
    age <- age[order(age)]
    sim.mat <- matrix(NA, nsim, nobs)
    for(i in 1:nobs) {
        sim.mat[,i] <- rnorm(nsim, age[i], error[i])
    }
    sim.res <- apply(sim.mat, 1, mytidy)

    sim.res <- list(
        simulations=sim.res,
        age=age,
        error=error,
        nsim=nsim)
    class(sim.res) <- "eventfreq"
    sim.res
}

eventfreq.formula <- function(formula, data=list(), nsim=10000, ...) {
    mf <- model.frame(formula=formula, data=data)
    error <- model.matrix(attr(mf, "terms"), data=mf)
    age <- model.response(mf)
    ## note age has intercept, until I get a nice
    ## way of dropping the intercept I just use
    ## second column
    sim.res <- eventfreq.default(as.numeric(age), as.numeric(error[,2]), nsim=nsim, ...)
    sim.res$call <- match.call()
    sim.res$formula <- formula
    sim.res
}

summary.eventfreq <- function(object, breaks=5,...) {
    ## a summary by period of the number of events
    ## work out the breaks
    ## Note calculating the expected frequency needs more thought
    
    dots = list(...)
    if(length(breaks)==1 & breaks[1]==1) {
        ## overall summary
        freq <- apply(object$simulations, 2,
                      FUN=function(x) {sum(is.finite(x))}
                      )
        eventfreq.df <- data.frame(
            mean=mean(freq, na.rm=TRUE),
            median=median(freq, na.rm=TRUE),
            lower= quantile(freq, 0.05, na.rm=TRUE),
            upper=quantile(freq,  0.95, na.rm=TRUE),
            period=paste("[",
                paste(round(range(object$simulations,na.rm=TRUE)),
                      collapse=","),"]")
        )
    } else {
        if(length(breaks)==1) {
            ## then provided a single number, so
            ## choose breaks accordingly
            ## recreate what cut does
            ageRange <- range(object$simulations,na.rm=TRUE)
            ageBreaks <- seq(ageRange[1],ageRange[2], length=breaks)
            ageBreaks[1] <- ageBreaks[1]*(1-0.001)
            ageBreaks[breaks] <- ageBreaks[breaks]*(1+0.001)
        } else {
            ageBreaks <- breaks
        }
        ##if you supplied a labels argument to the function
        ## use it, otherwise make one up
        if("labels" %in% names(dots)) {
            ageLabels <- dots$labels
            freq <- apply(object$simulations, 2, FUN=function(x){
                table(cut(x,breaks=ageBreaks,...))})
        } else {
            ageLabels <- levels(cut(object$simulations, breaks=ageBreaks))
            freq <- apply(object$simulations, 2, FUN=function(x){
                table(cut(x,breaks=ageBreaks,labels=ageLabels,...))})
        }
        eventfreq.df <- data.frame(
            mean=apply(freq, 1, mean, na.rm=TRUE),
            median=apply(freq, 1, median, na.rm=TRUE),
            lower=apply(freq, 1, quantile, 0.05, na.rm=TRUE),
            upper=apply(freq, 1, quantile, 0.95, na.rm=TRUE),
            period=factor(ageLabels, levels=ageLabels)
            )
    }
    class(eventfreq.df) <- c("data.frame", "summary.eventfreq")
    eventfreq.df
}

print.summary.eventfreq <- function(x, ...) {
    print(x)
}
    
