\name{eventfreq}
\alias{eventfreq}
\alias{eventfreq.default}
\alias{eventfreq.formula}
\alias{print.summary.eventfreq}
\alias{summary.eventfreq}
\title{Event Frequency under Uncertainty}
\description{Estimate event frequencies when date of event is measured
  with error, and observations may be from the same event.
}
\usage{
eventfreq(age, ...)
\method{eventfreq}{default}(age, error, nsim, ...)
\method{eventfreq}{formula}(formula, data, nsim, ...)
\method{summary}{eventfreq}(object, breaks, ...)
\method{print.summary}{eventfreq}(x, ...)
}
\arguments{
  \item{age}{ a numeric vector of ages, representing the estimate of the
  age of the sample. }
  \item{error}{ a numeric vector of the uncertainty around the age. Typically the standard
  deviation assuming a Gaussian distribution for uncertainty. }
  \item{formula}{ a formula, with the age on the LHS, and error on the
  right.}
  \item{data}{ an optional data.frame containing the variables.}
  \item{nsim}{ the number of simulations to draw from to determine
  frequencies. Default is 10000.}
  \item{breaks}{ The breaks used to summarise the frequencies. By
    default breaks=5, but can be a vector. See cut for details.}
  \item{object}{ An object of class eventfreq}
  \item{x}{A summary object from eventfreq}
  \item{...}{Other arguments that can be passed (presently only for
    summary in which they are passed to cut}
}
\value{
  An object of class eventfreq, a list containing
\item{simulations}{The number of simulations used (nsim)}
\item{age}{From the input.}
\item{error}{From the input.}
\item{simulations}{The simulations. A matrix with number of rows equal
  to the number of observations, and number of columns equal to the
  number of simulations. Element i,j is either NA, which means that
  event was captured by a different observation, or an age estimate.
}
}
\details{

  This approach builds largely on work of Parnell et. al. (2008). Their
  work explains the method of identifying synchroneity of two
  events. The approach in these functions is similar in that we first
  sample from the distribution of ages of each event. Events that fall
  out of order are considered to be duplicates, and so are ignored. At
  each realisation then, we can count the number of distinct events
  within a particular time interval. After many such draws, we are able
  to summarise the number of events, thus providing some information on
  the uncertainty of the estimates.

  Note that I have some reservations about this method for a number of
  reasons. Firstly, as Parnell et. al. (2008) point out, the ability to
  determine order of events works best when dealing with pairs of sites,
  ordering with multiple sites becomes more complex. Secondly, when the
  interval in which we wish to estimate the number of events is small,
  we end up with a very similar product to the summed Guassian
  distributions. The benefit of the eventfreq, perhaps, is that we are
  able to estimate the number of events, rather than provide probability
  densities. Finally, comparing the frequency of events over different
  intervals assumes that you have a reasonable sample over time. If
  preservation is an issue, then there will be an underestimate of the
  frequency of events at early dates.

  
}
\references{
  Parnell, A. C., Haslett, J., Allen, J. R. M., Buck, C. E. and Huntley,
  B. 2008, A flexible approach to assessing synchroneity of past events
  using Bayesian reconstructions of sedimentation history, Quaternary
  Science Reviews, 27, 1872-1885.
}
\seealso{
  pdfguass
}
\examples{
data(floodp)
sim.res <- eventfreq(Age.BP.AD ~ error, floodp)
periods <- seq(0,2000, by=500)
periodstr <- c("0-500","500-1000","1000-1500", "1500-2000")
print(sim.freq <- summary(sim.res, breaks=periods, labels=periodstr))

limits <- aes(ymax = upper, ymin=lower)
p <- ggplot(sim.freq, aes(y=mean, x=period))
p + geom_bar(stat="identity") +
    geom_errorbar(limits, width=0.25) +
    xlab("Period (years)") + ylab("Mean no. of events")
}
\author{Robert Denham}
\keyword{frequency}

